<?php
// ***************************************************************
// Date: 25/03/2005 - File name: inc_cg_template.php
//
// This is the Coolguys template engine class.
//
// It adds an object programming interface between the php files
// (the code) and the template file(s) (the design). It supports
// various macros including conditions, loops, includes and data
// transit (variables and arrays).
// ***************************************************************

class cg_template {

	var $class_data;
	var $path = NULL;

	function set_var($value) {
		$value_array = explode('==', $value, 2);
		$this->class_data[$value_array[0]] = $value_array[1];
	}

	function get_var($value) {
		return($this->class_data[$value]);
	}

	function push_array($value) {
		$value_array = explode('==', $value, 2);
		if(!is_array($this->class_data[$value_array[0]]))
			$this->class_data[$value_array[0]] = array();
		array_push($this->class_data[$value_array[0]], $value_array[1]);
	}

	function template_engine() {

		// ***************************************************************
		// This function parses "while", "set", "if",
		// "var", "inc" and "sys" macros
		// ***************************************************************

		if(!is_file($this->path))
			return('<h3>Template error: you must specify a valid template file (using $my_template_name->path) before running template_engine().</h3>');

		// Read file
		$template_file = file_get_contents($this->path);


		$pos = strpos($template_file, '{');

		while ($pos !== FALSE) {
			$len = strpos($template_file, '}', $pos + 4) - $pos + 1;
			$rest = explode('_', substr($template_file, $pos + 1, $len - 2));

			switch($rest[0]) {
			case 'while':
				$var_name = $rest[1];

				if (($pos2 = strpos($template_file, "{nwhile_$var_name}", $pos + $len)) === FALSE)
					return('<h3>Template error: all {while_var} type macros must be completed with {nwhile_var} macros.</h3>');

				// Remove both macros
				$template_file = substr_replace($template_file, '', $pos, $len);
				$template_file = substr_replace($template_file, '', $pos2 - $len, $len + 1);

				// Duplicate string
				$cp_str_len = $pos2 - $pos - $len;
				$cp_str = str_repeat(substr($template_file, $pos, $cp_str_len), $this->class_data[$var_name]);
				$template_file = substr_replace($template_file, $cp_str, $pos, $cp_str_len);
				break;
			
			case 'set':
				$str = ereg_replace('\$([0-9a-zA-z]+)', '$this->class_data[\'\\1\']', $rest[1]);
				eval("\$str = $str;");
				$template_file = substr_replace($template_file, '', $pos, $len);
				break;

			case 'if':
				$var_name = $rest[1];

				if (($pos2 = strpos($template_file, "{nif_$var_name}", $pos + $len)) === FALSE)
					return('<h3>Template error: all {if_var} type macros must be completed with {nif_var} macros.</h3>');

				if ($this->class_data[$var_name]) { // Just delete macros
					$template_file = substr_replace($template_file, '', $pos, $len);
					$template_file = substr_replace($template_file, '', $pos2 - $len, $len + 1);
				} else { // Delete whole section
					$template_file = substr_replace($template_file, '', $pos, $pos2 - $pos + $len + 1);
				}
				break;

			case 'var':
				if (is_array($this->class_data[$rest[1]]))
					$replace = array_pop($this->class_data[$rest[1]]);
				else
					$replace = $this->class_data[$rest[1]];
				$template_file = substr_replace($template_file, $replace, $pos, $len);
				$pos += strlen($replace);
				break;

			case 'inc':
				$include_file = file_get_contents($rest[1]);
				$template_file = substr_replace($template_file, $include_file, $pos, $len);
				break;

			case 'sys':
				switch ($rest[1]) {
				case 'date_long':
					$value = strftime('%A, %B %d');
					break;
				case 'date':
				case 'date_short':
					$value = strftime('%d/%m/%y');
					break;
				case 'time':
					$value = strftime('%R');
					break;
				default:
					$value = 'Invalid sys request.';
					break;
				}

				$template_file = substr_replace($template_file, $value, $pos, $len);
				break;

			default:
				// Prevents infinite loops should the template contain "{" char without it being a macro
				$pos++;
				break;
			}

			// Repeat
			$pos = strpos($template_file, '{', $pos);
		}

		return($template_file);
	}
}
?>